@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2011 Red Hat, Inc.
+ * Copyright (C) 2019 Canonical Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,6 +18,7 @@
  *
  * Authors:
  *        Colin Walters <walters@verbum.org>
+ *        Marco Trevisan <marco@ubuntu.com>
  */
 
 #include <config.h>
@@ -34,14 +36,14 @@
 
 #include "gsm-icon-names.h"
 
-#define GSM_FAIL_WHALE_DIALOG_GET_PRIVATE(o)                                \
-        (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSM_TYPE_FAIL_WHALE_DIALOG, GsmFailWhaleDialogPrivate))
-
-struct _GsmFailWhaleDialogPrivate
+struct _GsmFailWhaleDialog
 {
+        GtkWindow parent;
+
         gboolean debug_mode;
         gboolean allow_logout;
         gboolean extensions;
+        GdkMonitor *monitor;
         GdkRectangle geometry;
 };
 
@@ -81,40 +83,32 @@ _window_move_resize_window (GsmFailWhaleDialog *window,
                             gboolean  move,
                             gboolean  resize)
 {
-        if (window->priv->debug_mode)
+        if (window->debug_mode)
                 return;
 
         g_debug ("Move and/or resize window x=%d y=%d w=%d h=%d",
-                 window->priv->geometry.x,
-                 window->priv->geometry.y,
-                 window->priv->geometry.width,
-                 window->priv->geometry.height);
+                 window->geometry.x,
+                 window->geometry.y,
+                 window->geometry.width,
+                 window->geometry.height);
 
         if (resize) {
                 gtk_window_resize (GTK_WINDOW (window),
-                                   window->priv->geometry.width,
-                                   window->priv->geometry.height);
+                                   window->geometry.width,
+                                   window->geometry.height);
         }
 
         if (move) {
                 gtk_window_move (GTK_WINDOW (window),
-                                 window->priv->geometry.x,
-                                 window->priv->geometry.y);
+                                 window->geometry.x,
+                                 window->geometry.y);
         }
 }
 
 static void
 update_geometry (GsmFailWhaleDialog *fail_dialog)
 {
-        int monitor;
-        GdkScreen *screen;
-
-        screen = gtk_widget_get_screen (GTK_WIDGET (fail_dialog));
-        monitor = gdk_screen_get_primary_monitor (screen);
-
-        gdk_screen_get_monitor_geometry (screen,
-                                         monitor,
-                                         &fail_dialog->priv->geometry);
+        gdk_monitor_get_geometry (fail_dialog->monitor, &fail_dialog->geometry);
 }
 
 static void
@@ -164,24 +158,24 @@ gsm_fail_whale_dialog_size_request (GtkWidget      *widget,
 
         fail_dialog = GSM_FAIL_WHALE_DIALOG (widget);
 
-        old_geometry = fail_dialog->priv->geometry;
+        old_geometry = fail_dialog->geometry;
 
         update_geometry (fail_dialog);
 
-        requisition->width  = fail_dialog->priv->geometry.width;
-        requisition->height = fail_dialog->priv->geometry.height;
+        requisition->width  = fail_dialog->geometry.width;
+        requisition->height = fail_dialog->geometry.height;
 
         if (!gtk_widget_get_realized (widget)) {
                 return;
         }
 
-        if (old_geometry.width  != fail_dialog->priv->geometry.width ||
-            old_geometry.height != fail_dialog->priv->geometry.height) {
+        if (old_geometry.width  != fail_dialog->geometry.width ||
+            old_geometry.height != fail_dialog->geometry.height) {
                 size_changed = TRUE;
         }
 
-        if (old_geometry.x != fail_dialog->priv->geometry.x ||
-            old_geometry.y != fail_dialog->priv->geometry.y) {
+        if (old_geometry.x != fail_dialog->geometry.x ||
+            old_geometry.y != fail_dialog->geometry.y) {
                 position_changed = TRUE;
         }
 
@@ -252,15 +246,13 @@ gsm_fail_whale_dialog_class_init (GsmFailWhaleDialogClass *klass)
         widget_class->get_preferred_height = gsm_fail_whale_dialog_get_preferred_height;
         widget_class->get_preferred_width_for_height = gsm_fail_whale_dialog_get_preferred_width_for_height;
         widget_class->get_preferred_height_for_width = gsm_fail_whale_dialog_get_preferred_height_for_width;
-
-        g_type_class_add_private (klass, sizeof (GsmFailWhaleDialogPrivate));
 }
 
 static void
 on_logout_clicked (GtkWidget          *button,
                    GsmFailWhaleDialog *fail_dialog)
 {
-        if (!fail_dialog->priv->debug_mode) {
+        if (!fail_dialog->debug_mode) {
                 g_spawn_command_line_async ("gnome-session-quit --force", NULL);
         }
         gtk_main_quit ();
@@ -269,7 +261,6 @@ on_logout_clicked (GtkWidget          *button,
 static void
 setup_window (GsmFailWhaleDialog *fail_dialog)
 {
-        GsmFailWhaleDialogPrivate *priv;
         GtkWidget *box;
         GtkWidget *image;
         GtkWidget *label;
@@ -277,9 +268,10 @@ setup_window (GsmFailWhaleDialog *fail_dialog)
         GtkWidget *button_box;
         GtkWidget *button;
         GdkPixbuf *fail_icon;
+        GdkDisplay *display;
         char *markup;
-
-        priv = fail_dialog->priv;
+        int scale_factor;
+        int i;
 
         gtk_window_set_title (GTK_WINDOW (fail_dialog), "");
         gtk_window_set_icon_name (GTK_WINDOW (fail_dialog), GSM_ICON_COMPUTER_FAIL);
@@ -288,8 +280,19 @@ setup_window (GsmFailWhaleDialog *fail_dialog)
         gtk_window_set_keep_above (GTK_WINDOW (fail_dialog), TRUE);
         gtk_window_stick (GTK_WINDOW (fail_dialog));
         gtk_window_set_position (GTK_WINDOW (fail_dialog), GTK_WIN_POS_CENTER_ALWAYS);
+
         /* only works if there is a window manager which is unlikely */
-        gtk_window_fullscreen (GTK_WINDOW (fail_dialog));
+        display = gtk_widget_get_display (GTK_WIDGET (fail_dialog));
+        for (i = 0; i < gdk_display_get_n_monitors (display); i++) {
+                if (gdk_display_get_monitor (display, i) == fail_dialog->monitor) {
+                        GdkScreen *screen;
+
+                        screen = gtk_widget_get_screen (GTK_WIDGET (fail_dialog));
+                        gtk_window_fullscreen_on_monitor (GTK_WINDOW (fail_dialog),
+                                                          screen, i);
+                        break;
+                }
+        }
 
         box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 10);
         gtk_widget_set_valign (box, GTK_ALIGN_CENTER);
@@ -297,11 +300,13 @@ setup_window (GsmFailWhaleDialog *fail_dialog)
 
         gtk_container_add (GTK_CONTAINER (fail_dialog), box);
 
-        fail_icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-                                              GSM_ICON_COMPUTER_FAIL,
-                                              128,
-                                              0,
-                                              NULL);
+        scale_factor = gdk_monitor_get_scale_factor (fail_dialog->monitor);
+        fail_icon = gtk_icon_theme_load_icon_for_scale (gtk_icon_theme_get_default (),
+                                                        GSM_ICON_COMPUTER_FAIL,
+                                                        128,
+                                                        scale_factor,
+                                                        0,
+                                                        NULL);
         if (fail_icon != NULL) {
                 image = gtk_image_new_from_pixbuf (fail_icon);
                 gtk_widget_show (image);
@@ -316,9 +321,9 @@ setup_window (GsmFailWhaleDialog *fail_dialog)
         gtk_widget_show (label);
         gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
 
-        if (!priv->allow_logout)
+        if (!fail_dialog->allow_logout)
                 message_label = gtk_label_new (_("A problem has occurred and the system can’t recover. Please contact a system administrator"));
-        else if (priv->extensions)
+        else if (fail_dialog->extensions)
                 message_label = gtk_label_new (_("A problem has occurred and the system can’t recover. All extensions have been disabled as a precaution."));
         else
                 message_label = gtk_label_new (_("A problem has occurred and the system can’t recover.\nPlease log out and try again."));
@@ -335,7 +340,7 @@ setup_window (GsmFailWhaleDialog *fail_dialog)
         gtk_box_pack_end (GTK_BOX (box),
                           button_box, FALSE, FALSE, 0);
 
-        if (priv->allow_logout) {
+        if (fail_dialog->allow_logout) {
                 button = gtk_button_new_with_mnemonic (_("_Log Out"));
                 gtk_widget_show (button);
                 gtk_box_pack_end (GTK_BOX (button_box),
@@ -348,12 +353,58 @@ setup_window (GsmFailWhaleDialog *fail_dialog)
 static void
 gsm_fail_whale_dialog_init (GsmFailWhaleDialog *fail_dialog)
 {
-        fail_dialog->priv = GSM_FAIL_WHALE_DIALOG_GET_PRIVATE (fail_dialog);
 }
 
 static gboolean debug_mode = FALSE;
 static gboolean allow_logout = FALSE;
 static gboolean extensions = FALSE;
+static GList *dialogs = NULL;
+
+static void
+create_fail_dialog (GdkMonitor *monitor)
+{
+        GsmFailWhaleDialog *fail_dialog;
+
+        fail_dialog = g_object_new (GSM_TYPE_FAIL_WHALE_DIALOG, NULL);
+        fail_dialog->debug_mode = debug_mode;
+        fail_dialog->allow_logout = allow_logout;
+        fail_dialog->extensions = extensions;
+        fail_dialog->monitor = monitor;
+
+        setup_window (fail_dialog);
+
+        g_signal_connect (fail_dialog, "destroy",
+                          G_CALLBACK (gtk_main_quit), NULL);
+
+        gtk_widget_show (GTK_WIDGET (fail_dialog));
+
+        dialogs = g_list_prepend (dialogs, fail_dialog);
+}
+
+static void
+on_monitor_added (GdkDisplay  *display,
+                  GdkMonitor  *monitor)
+{
+        create_fail_dialog (monitor);
+}
+
+static void
+on_monitor_removed (GdkDisplay  *display,
+                    GdkMonitor  *monitor)
+{
+        GList *l;
+
+        for (l = dialogs; l;) {
+                GList *next = l->next;
+                GsmFailWhaleDialog *fail_dialog = l->data;
+
+                if (fail_dialog->monitor == monitor) {
+                        dialogs = g_list_delete_link (dialogs, l);
+                        gtk_widget_destroy (GTK_WIDGET (fail_dialog));
+                }
+                l = next;
+        }
+}
 
 int main (int argc, char *argv[])
 {
@@ -364,8 +415,9 @@ int main (int argc, char *argv[])
                 { NULL, 0, 0, 0, NULL, NULL, NULL }
         };
 
-        GsmFailWhaleDialog        *fail_dialog;
         GError *error = NULL;
+        GdkDisplay *display;
+        int i;
 
         bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -392,17 +444,15 @@ int main (int argc, char *argv[])
         if (g_strcmp0 (g_getenv ("RUNNING_UNDER_GDM"), "true") == 0)
                 allow_logout = FALSE;
 
-        fail_dialog = g_object_new (GSM_TYPE_FAIL_WHALE_DIALOG, NULL);
-        fail_dialog->priv->debug_mode = debug_mode;
-        fail_dialog->priv->allow_logout = allow_logout;
-        fail_dialog->priv->extensions = extensions;
+        display = gdk_display_get_default ();
+        for (i = 0; i < gdk_display_get_n_monitors (display); i++) {
+                create_fail_dialog (gdk_display_get_monitor (display, i));
+        }
 
-        setup_window (fail_dialog);
-
-        g_signal_connect (fail_dialog, "destroy",
-                          G_CALLBACK (gtk_main_quit), NULL);
-
-        gtk_widget_show (GTK_WIDGET (fail_dialog));
+        g_signal_connect (display, "monitor-added",
+                          G_CALLBACK (on_monitor_added), &dialogs);
+        g_signal_connect (display, "monitor-removed",
+                          G_CALLBACK (on_monitor_removed), &dialogs);
 
         gtk_main ();
 
