@@ -2,7 +2,6 @@
 
 import os
 import shutil
-import subprocess
 import sys
 
 if os.environ.get('DESTDIR'):
@@ -10,11 +9,6 @@ if os.environ.get('DESTDIR'):
 else:
   install_root = sys.argv[1]
 
-if not os.environ.get('DESTDIR'):
-  schemadir = os.path.join(install_root, 'glib-2.0', 'schemas')
-  print('Compile gsettings schemas...')
-  subprocess.call(['glib-compile-schemas', schemadir])
-
 # FIXME: Meson is unable to copy a generated target file:
 #        https://groups.google.com/forum/#!topic/mesonbuild/3iIoYPrN4P0
 dst_dir = os.path.join(install_root, 'wayland-sessions')
