@@ -127,10 +127,10 @@ do_restart_dbus (void)
                                              SYSTEMD_DBUS,
                                              SYSTEMD_PATH_DBUS,
                                              SYSTEMD_INTERFACE_DBUS,
-                                             "TryRestartUnit",
+                                             "StopUnit",
                                              g_variant_new ("(ss)",
                                                             "dbus.service",
-                                                            "replace"),
+                                                            "fail"),
                                              NULL,
                                              G_DBUS_CALL_FLAGS_NO_AUTO_START,
                                              -1, NULL, &error);
@@ -164,11 +164,13 @@ leader_fifo_io_cb (gint fd,
 
         sd_notify (0, "STOPPING=1");
 
-        if (condition | G_IO_IN) {
+        if (condition & G_IO_IN) {
                 char buf[1];
                 read (data->fifo_fd, buf, 1);
+                g_main_loop_quit (data->loop);
         }
-        if (condition | G_IO_HUP) {
+
+        if (condition & G_IO_HUP) {
                 g_main_loop_quit (data->loop);
         }
 
